/*
 * This program and the accompanying materials are made available under the terms of the
 * Eclipse Public License v2.0 which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-v20.html
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Copyright Contributors to the Zowe Project.
 */

/**
 * @file This script is used to generate the bundled package of this package. It
 * will go and add each dependency to the bundeledDependencies and save it to the
 * package.json file. The final line of output of this script will be the name
 * under which the archive should be saved. (IT IS NOT WHAT WAS GENERATED BY `npm pack`)
 */

const jsonfile = require("jsonfile");
const path = require("path");

const packageLocation = path.join(__dirname, "../package.json");

const packageJson = jsonfile.readFileSync(packageLocation);

// Grab the name and version for use later
let name = packageJson.name;
const version = packageJson.version;

// Initialize bundled dependencies to an empty object
packageJson.bundledDependencies = [];

// Add each dependency as a bundled dependency
for(const x in packageJson.dependencies) {
  packageJson.bundledDependencies.push(x);
}

console.log("====NEW PACKAGE.JSON====");
console.log(packageJson);
console.log("========================");

// Save back to package.json
jsonfile.writeFileSync(packageLocation, packageJson, {spaces: 2});

// We'll leave the actual command execution up to the jenkins process, but
// let's be nice and tell it what it will be named.
console.log("====ARCHIVE_NAME====");
if(name[0] === "@") {
  name = name.substring(1);
}

name = name.replace(/\//g, "-");
console.log(`${name}-${version}-bundled.tgz`);
